Program HPG2COM;
{*****************************************************************

     (C) 1990 California Institute of Technology
     Scott W. Wedge, Caltech MMIC Group

     Program for copying a file to a communications port.
     By altering the source code it is possible to
     specify data rate, parity, byte length, and number
     of start and stop bits.

     See Turbo Pascal Reference Guide, pages 212-216

******************************************************************}

Uses
     Crt,Dos;

Const
     endfile = -1;
     newline = 10;

Type
     character = -1..127;    {ASCII, plus endfile character}
     AuxRec = Record
                Port, Params: word;
                Unused: array[1..12] of byte;
              end;

Var
     Com1,f     : Text;
     Char_in    : Char;
     FileName   : String;
     c          : character;
     m          : integer;

{*********************************************************
 Using Procedure AssignAux:
     For COM1 use   AssignAux(Com1,0,$xx);
     For COM2 use   AssignAux(Com2,1,$xx);

     Shown above as $xx is the Parameter Byte

          Bit     7   6   5   4   3   2   1   0
                  |   |   |   |   |   |   |   |
                  |   |   |   |   |   |   |    -> Word Length select
                  |   |   |   |   |   |    ->  Word Length select
                  |   |   |   |   |    ->  Number of Stop bits
                  |   |   |   |   ->  Parity Enable
                  |   |   |    ->  Even parity select
                  |   |    -> Buad rate LSB
                  |    ->  Buad rate bit
                   ->  Baud rate MSB

            Bit 1    Bit 0    Word Length
               0       0          5
               0       1          6
               1       0          7
               1       1          8

            Bit 2  --  Stop Bits
               0   --  1
               1   --  1.5 or 2

            Bits  4   3    --Parity--
                  X   0      None
                  0   1      Odd
                  1   1      Even

            Bits  7  6  5   --Baud Rate--

                  0  0  0      110
                  0  0  1      150
                  0  1  0      300
                  0  1  1      600
                  1  0  0     1200
                  1  0  1     2400
                  1  1  0     4800
                  1  1  1     9600


For more info see RS-232 technical reference page 5-125

*********************************************************}

{$R-,S-}

Procedure AuxInit(Port, Params: word);
inline(
     $58/                    { POP AX     ; Pop parameters }
     $5A/                    { POP DX     ; Pop port number }
     $B4/$00/                { MOV AH,0   ; Code for initialization }
     $CD/$14);               { INT 14H    ; Call BIOS }

Function AuxInChar(Port: word): char;
inline(
     $5A/                    { POP DX     ; Pop port number }
     $B4/$02/                { MOV AH,2   ; Code for input }
     $CD/$14);               { INT 14H    ; Call BIOS }

Procedure AuxOutChar(Port: word; Ch: char);
inline(
     $58/                    { POP AX     ; Pop character }
     $5A/                    { POP DX     ; Pop port number }
     $B4/$01/                { MOV AH,1   ; Code for output }
     $CD/$14);               { INT 14H    ; Call BIOS }

Function AuxInReady(Port: word): boolean;
inline(
     $5A/                    { POP DX     ; Pop port number }
     $B4/$03/                { MOV AH,3   ; Code for status }
     $CD/$14/                { INT 14H    ; Call BIOS }
     $88/$E0/                { MOV AL,AH  ; Get line status in AH }
     $24/$01);               { AND AL,1   ; Isolate Data Ready bit }

{$F+}

        {***********************************************************}

Function AuxInput(var F: TextRec): integer;
var
     P: word;
Begin
  with F,AuxRec(UserData) do begin
     P := 0;
     while AuxInReady(Port) and (P<BufSize) do begin
          BufPtr^[P] := AuxInChar(Port);
          Inc(P);
     end;
     BufPos := 0;
     BufEnd := P;
  end;
  AuxInput := 0;
end;

Function AuxOutput(var F: TextRec): integer;
var
     P: word;
Begin
  with F,AuxRec(UserData) do begin
     P := 0;
     while P<BufPos do begin
          AuxOutChar(Port,BufPtr^[P]);
          Inc(P);
     end;
     BufPos := 0;
  end;
  AuxOutput := 0;
end;

Function AuxIgnore(var F: TextRec): integer;
begin
     AuxIgnore := 0;
end;

Function AuxOpen(var F: TextRec): integer;
Begin
     with F,AuxRec(UserData) do begin
          AuxInit(Port,Params);
          if Mode=fmInput then begin
               InOutFunc := @AuxInput;
               FlushFunc := @AuxIgnore;
          end
          else begin
               Mode := fmOutput;
               InOutFunc := @AuxOutput;
               FlushFunc := @AuxOutput;
          end;
          CloseFunc := @AuxIgnore;
     end;
     AuxOpen := 0;
end;

{$F-}

{***********************************************************}

Procedure AssignAux(var F: Text; Port,Params: word);
{*
        Main procedure for using a file name to
        specify a communication port.
*}
Begin
     with TextRec(F) do begin
          Handle:= $FFFF;
          Mode := fmClosed;
          BufSize := Sizeof(Buffer);
          BufPtr := @Buffer;
          OpenFunc := @AuxOpen;
          AuxRec(UserData).Port := Port;
          AuxRec(UserData).Params := Params;
          Name[0] := #0;
     end;
end;

{**************************************************************}

Function getc(var c : character) : character;
{*
   Get one character from HPGL File
*}
var
   ch : char;

begin
     if (eof(f)) then
          c:=endfile
     else if (eoln(f)) then begin
          readln(f);
          c:=newline
     end
     else begin
          read(f,ch);
          c:=ord(ch)
     end;
     getc:=c
end;

{****************************************************}

procedure putc(c : character);
{*
    Send one character to serial port
*}
begin
     if (c = newline) then
        writeln(Com1)
     else
        write(Com1,chr(c))
end;

{***************************************************}

Function Device_Ready : Boolean;
{*
     Use BIOS interrupt to read status of COM1 port.
     COM1 is designated as Port=0 in DX, serial port
     status is given by interrupt 14H with AH=3H.
     Status is returned in AX.

     A value of "True" is returned only if the COM1
     Clear-to-send bit (AL, bit 4) is set.

*}
Var
   int_results   : registers;
   Clear_to_Send : byte;

Begin
    Device_Ready:= False;
    int_results.ah:=$03;  {serial port status}
    int_results.dx:=0;    {Port=0}
    intr($14,Dos.Registers(int_results));   {call BIOS interrupt}
    Clear_to_Send:= int_results.al AND $10;
    if Clear_to_Send=$10 then Device_Ready:=True;
End;


{****************************************************}


Begin   {** MAIN **}
     Clrscr;
     Textcolor(yellow);
     WriteLn;
     WriteLn('   HP-GL File -> COM1 Serial Data Transfer Utility.');
     WriteLn('   Version 2.0 (C) 1990 California Institute of Technology.');
     WriteLn('   Scott W. Wedge, Caltech MMIC Group');
     WriteLn;
     WriteLn('   COM1 parameters are set automatically.');
     WriteLn('   Use the following plotter parameters:');
     WriteLn('   9600 baud, no parity, D/Y=D.');
     WriteLn;
     FileName:=ParamStr(1);  {Get filename off command-line}
     if FileName='' then begin
       Write('   Enter HP-GL file-name (.HPG): ');
       Readln(FileName);
       WriteLn;
     end;
     if Pos('.',FileName)=0 then FileName:=FileName+'.HPG';
     WriteLn('   Searching for file ',FileName);
     WriteLn;
     Assign(f, FileName);
     {$I-} Reset(f); {$I+}  {open HP-GL file}
     if IOResult=0 then begin
       AssignAux(Com1,0,$E3);  {9600 baud, no parity, 8 bits }
       Rewrite(Com1);
       if Device_Ready then begin  {Device_Ready checks COM1 status}
         Write('   Working ');
         m:=0;
         while (getc(c) <> endfile) do begin
           putc(c);
           m:=m+1;
           if ((m mod 20) = 0) then write('.');
         end;
         WriteLn;
         WriteLn;
         WriteLn('   Transfer Complete');
         Close(f);
         Close(Com1);
       end
       else begin
         WriteLn('   Plotter off-line');
         WriteLn;
         Close(f);
         Close(Com1);
       end;
     end
     else begin
       WriteLn('   File not found');
       WriteLn;
     end;    {if IOResult}
End.